@@ -60,12 +60,14 @@ class VisListener(Listener):
 
 class Torrent(object):
 
-    def __init__(self, torrent_path, directory='', port=55308):
+    def __init__(self, torrent_path, directory='', port=55308,
+                 download_all=False):
         torrent_dict = tparser.bdecode_file(torrent_path)
         self.torrent_dict = torrent_dict
         self.peer_dict = {}
         self.peer_ips = []
         self.port = port
+        self.download_all = download_all
         self.r = None
         self.tracker_response = None
         self.peer_dict = {}
@@ -77,12 +79,33 @@ class Torrent(object):
                                     VisListener(torrent=self, port=8035)])
         if directory:
             os.chdir(directory)
-        dirname = self.torrent_dict['info']['name']
+        if 'files' in self.torrent_dict['info']:
+            dirname = self.torrent_dict['info']['name']
+        else:
+            dirname = None
+        file_list = []
+
+        # Multifile case
+        if 'files' in self.torrent_dict['info']:
+            file_list.extend(self.torrent_dict['info']['files'])
+            multifile = True
+
+        # Deal with single-file torrents by building up the kind of dict
+        # found in torrent_dict['info']['files']
+        elif 'name' in self.torrent_dict['info']:
+            info_dict = {}
+            info_dict['path'] = self.torrent_dict['info']['name']
+            info_dict['length'] = self.torrent_dict['info']['length']
+            file_list.append(info_dict)
+            multifile = False
+        else:
+            raise Exception('Invalid .torrent file')
+
         self.switchboard = Switchboard(dirname=dirname,
-                                       file_list=self.torrent_dict
-                                       ['info']['files'], piece_length=
+                                       file_list=file_list, piece_length=
                                        self.piece_length, num_pieces=
-                                       self.num_pieces)
+                                       self.num_pieces, multifile=multifile,
+                                       download_all=download_all)
 
     @property
     def piece_length(self):
@@ -98,7 +121,11 @@ class Torrent(object):
 
     @property
     def length(self):
-        return sum([i['length'] for i in self.torrent_dict['info']['files']])
+        if 'files' in self.torrent_dict['info']:
+            return sum([i['length'] for i in
+                       self.torrent_dict['info']['files']])
+        else:
+            return self.torrent_dict['info']['length']
 
     @property
     def last_piece_length(self):
@@ -259,11 +286,16 @@ def main():
     argparser.add_argument('-d', '--directory', nargs='?', default=DEFAULT_DIR,
                            help=('Where to save downloaded files. Default '
                                  'is {}'.format(DEFAULT_DIR)))
+    argparser.add_argument('-a', '--all', action='store_true',
+                           help=('Set flag to download all files in torrent'),
+                           default=False)
     args = argparser.parse_args()  # Getting path from command line
     torrent_path = get_path(args.torrent_path)
     directory = get_path(args.directory)
     port = args.port
-    mytorrent = Torrent(torrent_path, directory=directory, port=port)
+    download_all = args.all
+    mytorrent = Torrent(torrent_path, directory=directory, port=port,
+                        download_all=download_all)
     with mytorrent:
         mytorrent.tracker_request()
         mytorrent.handshake_peers()
